var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { useEffect, useState } from 'react';
import { useApiContext, ApiContextProvider } from './CacheContext';
import { areObjectsEqual, generateRequestHeaders } from './utils';
/*
Usage:
GET
const {data, isLoading, isError, error} = useApi<ResponseType, PostDataType>({
    apiId: 'some-api'
    apiUrl: 'https://api.example.com',
    method: Method.GET
})

POST, PUT, PATCH
const {data, isLoading, isError, error} = useApi<ResponseType, PostDataType>({
    apiId: 'some-api'
    apiUrl: 'https://api.example.com',
    method: Method.POST,
    data: {firstName: "Test"}
})
 */
var Method;
(function (Method) {
    Method["GET"] = "GET";
    Method["POST"] = "POST";
    Method["PUT"] = "PUT";
    Method["PATCH"] = "PATCH";
    Method["DELETE"] = "DELETE";
})(Method || (Method = {}));
var useApi = function (_a) {
    var apiId = _a.apiId, apiUrl = _a.apiUrl, method = _a.method, data = _a.data, headers = _a.headers, cacheExpiry = _a.cacheExpiry, retry = _a.retry;
    var _b = useApiContext(), getCache = _b.getCache, setCache = _b.setCache;
    var retryTimes = retry || 0;
    var _c = useState({
        data: undefined,
        error: null,
        isError: false,
        isLoading: true,
        isRetrying: false,
    }), state = _c[0], setState = _c[1];
    var apiIdentifier = apiId || JSON.stringify({ apiUrl: apiUrl, method: method, data: data });
    var requestHeaders = generateRequestHeaders(headers);
    var cachedVsNewData = function (cachedData, response) {
        response.json().then(function (newData) {
            if (!areObjectsEqual(cachedData, newData)) {
                setState({
                    data: newData,
                    error: null,
                    isError: false,
                    isLoading: false,
                    isRetrying: false,
                });
            }
        });
    };
    var triggerAPI = function () { return __awaiter(void 0, void 0, void 0, function () {
        var cachedResponse, response, response, responseData, error_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 6, , 7]);
                    cachedResponse = getCache(apiIdentifier);
                    if (!cachedResponse) return [3 /*break*/, 2];
                    setState({
                        data: cachedResponse,
                        error: null,
                        isError: false,
                        isLoading: false,
                        isRetrying: false,
                    });
                    return [4 /*yield*/, fetch(apiUrl, {
                            method: method,
                            body: JSON.stringify(data),
                            headers: requestHeaders,
                        })];
                case 1:
                    response = _a.sent();
                    cachedVsNewData(cachedResponse, response);
                    return [3 /*break*/, 5];
                case 2: return [4 /*yield*/, fetch(apiUrl, {
                        method: method,
                        body: JSON.stringify(data),
                        headers: headers,
                    })];
                case 3:
                    response = _a.sent();
                    return [4 /*yield*/, response.json()];
                case 4:
                    responseData = _a.sent();
                    setState({
                        data: responseData,
                        error: null,
                        isError: false,
                        isLoading: false,
                        isRetrying: false,
                    });
                    setCache(apiIdentifier, responseData, cacheExpiry);
                    _a.label = 5;
                case 5: return [3 /*break*/, 7];
                case 6:
                    error_1 = _a.sent();
                    if (retryTimes && retryTimes > 0) {
                        retryTimes--;
                        setState({
                            data: undefined,
                            error: null,
                            isError: false,
                            isLoading: true,
                            isRetrying: true,
                        });
                        triggerAPI();
                    }
                    else {
                        setState({
                            data: undefined,
                            error: error_1,
                            isLoading: false,
                            isError: true,
                            isRetrying: false,
                        });
                    }
                    return [3 /*break*/, 7];
                case 7: return [2 /*return*/];
            }
        });
    }); };
    useEffect(function () {
        setState({
            data: undefined,
            error: null,
            isError: false,
            isLoading: true,
            isRetrying: false,
        });
        triggerAPI();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [apiId]);
    return state;
};
export { useApi, Method, ApiContextProvider };
//# sourceMappingURL=index.js.map